@@ -34,6 +34,7 @@
 #include <sys/param.h>
 #include <sys/stat.h>
 
+#include <linux/icmpv6.h>
 #include <linux/if_addr.h>
 #include <linux/if_link.h>
 #include <linux/if_packet.h>
@@ -820,13 +821,11 @@ link_neigh(struct dhcpcd_ctx *ctx, __unused struct interface *ifp,
 	rta = RTM_RTA(r);
 	len = RTM_PAYLOAD(nlm);
         if (r->ndm_family == AF_INET6) {
-		bool reachable;
+		bool unreachable;
 		struct in6_addr addr6;
 
-		reachable = (nlm->nlmsg_type == RTM_NEWNEIGH &&
-		    r->ndm_state &
-		    (NUD_REACHABLE | NUD_STALE | NUD_DELAY | NUD_PROBE |
-		     NUD_PERMANENT));
+		unreachable = (nlm->nlmsg_type == RTM_NEWNEIGH &&
+		    r->ndm_state & NUD_FAILED);
 		memset(&addr6, 0, sizeof(addr6));
 		for (; RTA_OK(rta, len); rta = RTA_NEXT(rta, len)) {
 			switch (rta->rta_type) {
@@ -836,7 +835,7 @@ link_neigh(struct dhcpcd_ctx *ctx, __unused struct interface *ifp,
 				break;
 			}
 		}
-		ipv6nd_neighbour(ctx, &addr6, reachable);
+		ipv6nd_neighbour(ctx, &addr6, !unreachable);
 	}
 
 	return 0;
@@ -1063,6 +1062,14 @@ add_attr_l(struct nlmsghdr *n, unsigned short maxlen, unsigned short type,
 	return 0;
 }
 
+static int
+add_attr_8(struct nlmsghdr *n, unsigned short maxlen, unsigned short type,
+    uint8_t data)
+{
+
+	return add_attr_l(n, maxlen, type, &data, sizeof(data));
+}
+
 static int
 add_attr_32(struct nlmsghdr *n, unsigned short maxlen, unsigned short type,
     uint32_t data)
@@ -1506,6 +1513,26 @@ if_route(unsigned char cmd, const struct rt *rt)
 			    RTA_DATA(metrics),
 			    (unsigned short)RTA_PAYLOAD(metrics));
 		}
+
+		if (rt->rt_dflags & RTDF_RA) {
+			uint8_t pref;
+
+			switch(rt->rt_pref) {
+			case RTPREF_LOW:
+				pref = ICMPV6_ROUTER_PREF_LOW;
+				break;
+			case RTPREF_MEDIUM:
+				pref = ICMPV6_ROUTER_PREF_MEDIUM;
+				break;
+			case RTPREF_HIGH:
+				pref = ICMPV6_ROUTER_PREF_HIGH;
+				break;
+			default:
+				pref = ICMPV6_ROUTER_PREF_INVALID;
+				break;
+			}
+			add_attr_8(&nlm.hdr, sizeof(nlm), RTA_PREF, pref);
+		}
 	}
 
 	if (!sa_is_loopback(&rt->rt_gateway))
@@ -1880,14 +1907,7 @@ struct nlml
 	char buffer[32];
 };
 
-static int
-add_attr_8(struct nlmsghdr *n, unsigned short maxlen, unsigned short type,
-    uint8_t data)
-{
-
-	return add_attr_l(n, maxlen, type, &data, sizeof(data));
-}
-
+#ifdef HAVE_IN6_ADDR_GEN_MODE_NONE
 static struct rtattr *
 add_attr_nest(struct nlmsghdr *n, unsigned short maxlen, unsigned short type)
 {
@@ -1904,6 +1924,7 @@ add_attr_nest_end(struct nlmsghdr *n, struct rtattr *nest)
 
 	nest->rta_len = (unsigned short)((char *)NLMSG_TAIL(n) - (char *)nest);
 }
+#endif
 
 static int
 if_disable_autolinklocal(struct dhcpcd_ctx *ctx, unsigned int ifindex)
