@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * Socket Address handling for dhcpcd
- * Copyright (c) 2015-2019 Roy Marples <roy@marples.name>
+ * Copyright (c) 2015-2020 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -288,11 +288,9 @@ sa_toprefix(const struct sockaddr *sa)
 #ifndef NDEBUG
 	/* Ensure the calculation is correct */
 	if (!sa_inprefix) {
-		union sa_ss ss;
+		union sa_ss ss = { .sa.sa_family = sa->sa_family };
 
 		sa_inprefix = true;
-		memset(&ss, 0, sizeof(ss));
-		ss.sa.sa_family = sa->sa_family;
 		sa_fromprefix(&ss.sa, prefix);
 		assert(sa_cmp(sa, &ss.sa) == 0);
 		sa_inprefix = false;
